#include "chara.h"

#include "sdl.h"
#include "floor.h"
#include "sprite.h"
#include "tileset.h"

chara::chara(class sdl * sdl, std::string filename, unsigned int tilesize, int x, int y) :
    x(x),
    y(y),
    step(0),
    dir(direction::SOUTH)
{
    this->anim = new tileset(sdl, filename, tilesize);
}

chara::~chara() {
    delete(this->anim);
}

void chara::draw(class sdl * sdl) {
    this->draw(sdl, 0, 0);
}

void chara::draw(class sdl * sdl, int x_shift, int y_shift) {
    struct sprite * sprite = this->anim->get(this->dir*2 + this->step);
    sprite->draw(sdl, this->x * sprite->w + x_shift, this->y * sprite->h + y_shift);
}

bool chara::move(class sdl * sdl, struct floor * floor) {
    int x_shift = 0;
    int y_shift = 0;
    int next_x;
    int next_y;

    if(sdl->keydown(SDL_SCANCODE_UP)) {
        y_shift--;
    }
    if(sdl->keydown(SDL_SCANCODE_DOWN)) {
        y_shift++;
    }
    if(sdl->keydown(SDL_SCANCODE_LEFT)) {
        x_shift--;
    }
    if(sdl->keydown(SDL_SCANCODE_RIGHT)) {
        x_shift++;
    }

    if(x_shift != 0 || y_shift != 0) {
        next_x = this->x + x_shift;
        next_y = this->y + y_shift;

        if(x_shift > 0) {
            this->dir = direction::EAST;
        } else if(x_shift < 0) {
            this->dir = direction::WEST;
        } else if(y_shift > 0) {
            this->dir = direction::SOUTH;
        } else if(y_shift < 0) {
            this->dir = direction::NORTH;
        }

        if(next_x >= 0 && next_x < floor->w && next_y >= 0 && next_y < floor->h) {
            if(floor->get(next_x, next_y) < tile_type::pillar_top) {
                this->x = next_x;
                this->y = next_y;
                this->step = (this->step + 1) % 2;
            }
        }

        return(true);
    } else {
        return(false);
    }
}
