#include "floor.h"

#include "sdl.h"
#include "tileset.h"
#include "sprite.h"

#include <cstdlib>
#include <iostream>
#include <fstream>

floor::floor(unsigned int w, unsigned int h) :
    tiles(),
    w(w),
    h(h)
{
    this->tiles.resize(w*h, 0);
}

void floor::set(unsigned int x, unsigned int y, unsigned int id) {
    this->tiles.at(y * this->w + x) = id;
}

unsigned int floor::get(unsigned int x, unsigned int y) {
    return(this->tiles.at(y * this->w + x));
}

void floor::load(std::string filename) {
    std::string line;
    std::ifstream file(filename);
    unsigned int nb_line = 0;
    enum tile_type t;

    if(file.is_open()) {
        for(nb_line = 0; getline(file, line) && nb_line < this->h; nb_line++) {
            for(unsigned int i=0; i<line.size() && i<this->w; i++) {
                switch(line.at(i)) {
                    case ' ': t = tile_type::soil_a; break;
                    case '.': t = tile_type::path; break;
                    case '#': t = tile_type::mosaic_a; break;
                    case '/': t = tile_type::mosaic_white; break;
                    case '\\':t = tile_type::mosaic_black; break;
                    case 'T': t = tile_type::wall_top; break;
                    case 'I': t = tile_type::wall; break;
                    case 'M': t = tile_type::wall_bot; break;
                    case 'H': t = tile_type::wall_top_vertical; break;
                    case 'O': t = tile_type::wall_top_round; break;
                    case 't': t = tile_type::wall_top_roof; break;
                    case 'i': t = tile_type::wall_alt_a; break;
                    case 'm': t = tile_type::wall_bot_alt_a; break;
                    case 'j': t = tile_type::wall_special; break;
                    case 'n': t = tile_type::wall_bot_special; break;
                    case '[': t = tile_type::wall_botlft; break;
                    case ']': t = tile_type::wall_botrgt; break;
                    case '(': t = tile_type::wall_lft; break;
                    case ')': t = tile_type::wall_rgt; break;
                    case '{': t = tile_type::wall_toplft; break;
                    case '}': t = tile_type::wall_toprgt; break;
                    case '+': t = tile_type::pillar_top; break;
                    case '-': t = tile_type::pillar; break;
                    case '=': t = tile_type::pillar_bot; break;
                    case 'x': t = tile_type::block_a; break;
                    default : t = tile_type::palette_a; break;
                }
                this->set(i, nb_line, t);
            }
        }
    }
    this->soften();
}

void floor::fill(unsigned int id) {
    for(unsigned int i=0; i<this->tiles.size(); i++) {
        this->tiles.at(i) = id;
    }
}

void floor::fill(int left, int top, int right, int bot, unsigned int id) {
    for(int x=left; x<=right; x++) {
        for(int y=top; y<=bot; y++) {
            this->set(x, y, id);
        }
    }
}

void floor::fill_rand(int left, int top, int right, int bot) {
    this->fill(left, top, right, bot, tile_type::soil_a);
}

void floor::fill_path(int left, int top, int right, int bot) {
    this->fill(left, top, right, bot, tile_type::path);
}

void floor::fill_wall(int left, int top, int right, int bot) {
    this->fill(left, top, right, bot, tile_type::wall);
}

void floor::soften() {
    struct floor * save = new floor(*this);
    bool left, right, up, down;

    for(int x=0; x<this->w; x++) {
        for(int y=0; y<this->h; y++) {
            if(save->get(x,y) == tile_type::path) {
                // Get edges
                left = right = up = down = false;
                if(x > 0 && save->get(x-1, y) != tile_type::path)
                    left = true;
                if(x < this->w-1 && save->get(x+1, y) != tile_type::path)
                    right = true;
                if(y > 0 && save->get(x, y-1) != tile_type::path)
                    up = true;
                if(y < this->h-1 && save->get(x, y+1) != tile_type::path)
                    down = true;
                // Smooth accordingly to edges
                if(!up && !down && !left && !right) {
                    if(rand()%10 == 0) {
                        this->set(x, y,
                            rand()%2 ?
                            tile_type::path_rare_a:
                            tile_type::path_rare_b);
                    }
                } else if(up && down && left && right) {
                    this->set(x, y, tile_type::path_lone);

                } else if(up && !down && !left && !right) {
                    this->set(x, y, tile_type::path_top);
                } else if(!up && down && !left && !right) {
                    this->set(x, y, tile_type::path_bot);
                } else if(!up && !down && left && !right) {
                    this->set(x, y, tile_type::path_lft);
                } else if(!up && !down && !left && right) {
                    this->set(x, y, tile_type::path_rgt);

                } else if(!up && down && left && right) {
                    this->set(x, y, tile_type::path_endbot);
                } else if(up && !down && left && right) {
                    this->set(x, y, tile_type::path_endtop);
                } else if(up && down && !left && right) {
                    this->set(x, y, tile_type::path_endrgt);
                } else if(up && down && left && !right) {
                    this->set(x, y, tile_type::path_endlft);

                } else if(!up && !down && left && right) {
                    this->set(x, y, tile_type::path_vertical);
                } else if(up && down && !left && !right) {
                    this->set(x, y, tile_type::path_horizontal);

                } else if(!up && down && !left && right) {
                    this->set(x, y, tile_type::path_botrgt);
                } else if(!up && down && left && !right) {
                    this->set(x, y, tile_type::path_botlft);
                } else if(up && !down && !left && right) {
                    this->set(x, y, tile_type::path_toprgt);
                } else if(up && !down && left && !right) {
                    this->set(x, y, tile_type::path_toplft);

                /*
                if(up && down && left && right) {
                    this->set(x, y, tile_type::path_lone);
                } else if(up && down && !left && !right) {
                    this->set(x, y, tile_type::path_vertical);
                } else if(!up && !down && left && right) {
                    this->set(x, y, tile_type::path_horizontal);
                } else if(up) {
                    if(left) {
                        this->set(x, y, tile_type::path_toplft);
                    } else if(right) {
                        this->set(x, y, tile_type::path_toprgt);
                    } else {
                        this->set(x, y, tile_type::path_top);
                    }
                } else if(down) {
                    if(left) {
                        this->set(x, y, tile_type::path_botlft);
                    } else if(right) {
                        this->set(x, y, tile_type::path_botrgt);
                    } else {
                        this->set(x, y, tile_type::path_bot);
                    }
                } else if(left) {
                    this->set(x, y, tile_type::path_lft);
                } else if(right) {
                    this->set(x, y, tile_type::path_rgt);
                } else {
                    if(rand()%10 == 0) {
                        this->set(x, y,
                            rand()%2 ?
                            tile_type::path_rare_a:
                            tile_type::path_rare_b);
                    }
                */
                }
/* XXX //
            } else if(save->get(x,y) == tile_type::wall) {
                // Get edges
                left = right = up = down = false;
                if(x > 0 && save->get(x-1, y) != tile_type::wall)
                    left = true;
                if(x < this->w-1 && save->get(x+1, y) != tile_type::wall)
                    right = true;
                if(y > 0 && save->get(x, y-1) != tile_type::wall)
                    up = true;
                if(y < this->h-1 && save->get(x, y+1) != tile_type::wall)
                    down = true;
                // Smooth accordingly to edges
                if(left && right) {
                    // Pillar
                    if(up) {
                        this->set(x, y, tile_type::pillar_top);
                    } else if(down) {
                        this->set(x, y, tile_type::pillar_bot);
                    } else {
                        this->set(x, y, tile_type::pillar);
                    }
                } else if(up) {
                    if(left) {
                        this->set(x, y, tile_type::wall_toplft);
                    } else if(right) {
                        this->set(x, y, tile_type::wall_toprgt);
                    } else {
                        this->set(x, y, tile_type::wall_top);
                    }
                } else if(down) {
                    if(left) {
                        this->set(x, y, tile_type::wall_botlft);
                    } else if(right) {
                        this->set(x, y, tile_type::wall_botrgt);
                    } else {
                        this->set(x, y, tile_type::wall_bot);
                    }
                } else if(left) {
                    this->set(x, y, tile_type::wall_lft);
                } else if(right) {
                    this->set(x, y, tile_type::wall_rgt);
                }
// XXX */
            } else if(save->get(x,y) == tile_type::soil_a) {
                switch(rand()%4) {
                    case 0: this->set(x, y, tile_type::soil_b); break;
                    case 1: this->set(x, y, tile_type::soil_c); break;
                    case 2: this->set(x, y, tile_type::soil_d); break;
                    case 3: break;
                }
            } else if(save->get(x,y) == tile_type::mosaic_a) {
                if(rand()%10 == 0) {
                    this->set(x, y, tile_type::mosaic_broken);
                } else {
                    switch(rand()%3) {
                        case 0: this->set(x, y, tile_type::mosaic_b); break;
                        case 1: this->set(x, y, tile_type::mosaic_c); break;
                        case 2: break;
                    }
                }
            } else if(save->get(x,y) == tile_type::wall_alt_a) {
                if(rand()%2) {
                    this->set(x, y, tile_type::wall_alt_b);
                }
            } else if(save->get(x,y) == tile_type::wall_bot_alt_a) {
                if(rand()%2) {
                    this->set(x, y, tile_type::wall_bot_alt_b);
                }
            } else if(save->get(x,y) == tile_type::block_a) {
                if(rand()%2) {
                    this->set(x, y, tile_type::block_b);
                }
            }
        }
    }
    delete(save);
}

void floor::draw(class sdl * sdl, struct tileset * tileset) {
    this->draw(sdl, tileset, 0, 0);
}

void floor::draw(class sdl * sdl, struct tileset * tileset, int x_shift, int y_shift) {
    for(unsigned int x=0; x<this->w; x++) {
        for(unsigned int y=0; y<this->h; y++) {
            struct sprite * sprite = tileset->get(this->get(x,y));
            sprite->draw(sdl, x*sprite->w + x_shift, y*sprite->h + y_shift);
        }
    }
}
