#include "sdl.h"
#include "sprite.h"
#include "tileset.h"
#include "floor.h"
// #include "object.h"
#include "chara.h"
#include "portal.h"

#include <cstdlib>
#include <ctime>
#include <list>
#include <vector>
#ifdef DEBUG
#include <iostream>
#endif // DEBUG

#define SCREEN_W 800
#define SCREEN_H 600
#define TILE_SIZE 20
#define GRID_W ((SCREEN_W/TILE_SIZE)/2) // 20
#define GRID_H (SCREEN_H/TILE_SIZE) // 30

/*
void cycle_tileset(std::list<struct tileset *>::iterator &it, std::list<struct tileset *> &lst) {
    it++;
    if(it == lst.end()) {
        it = lst.begin();
    }
}
*/

int main(int argc, char ** argv) {
    class sdl * my_sdl = new sdl(SCREEN_W, SCREEN_H);
    std::vector<struct tileset *> tilesets;
    unsigned int left_tileset, right_tileset;
    struct floor * test_floor = new floor(GRID_W, GRID_H);
    std::list<struct portal *> portals;
    // std::list<struct object *> objects;
    // struct object * player;
    struct chara * player;
    bool stop = false;

    srand(time(NULL));
    my_sdl->set_title("LD#30");
    tilesets.push_back(new tileset(my_sdl, "data\\tileset1.bmp", TILE_SIZE));
    tilesets.push_back(new tileset(my_sdl, "data\\tileset2.bmp", TILE_SIZE));
    tilesets.push_back(new tileset(my_sdl, "data\\tileset3.bmp", TILE_SIZE));
    tilesets.push_back(new tileset(my_sdl, "data\\tileset4.bmp", TILE_SIZE));
    tilesets.push_back(new tileset(my_sdl, "data\\tileset5.bmp", TILE_SIZE));
    tilesets.push_back(new tileset(my_sdl, "data\\tileset6.bmp", TILE_SIZE));
    tilesets.push_back(new tileset(my_sdl, "data\\tileset7.bmp", TILE_SIZE));
    tilesets.push_back(new tileset(my_sdl, "data\\tileset8.bmp", TILE_SIZE));
    tilesets.push_back(new tileset(my_sdl, "data\\tileset9.bmp", TILE_SIZE));
    left_tileset = 0;
    right_tileset = 1;

/*
    test_floor->fill(tile_type::path);
    test_floor->fill_rand(0, 0, test_floor->w-1, test_floor->h-1);
    test_floor->fill_wall(4, 1, 16, 3);
    test_floor->fill_path(4, 9, 16, 12);
    test_floor->fill_path(6, 13, 7, 17);
    test_floor->fill_path(14, 13, 14, 17);
    test_floor->fill_path(4, 18, 16, 20);

    test_floor->fill_path(8, 23, 12, 27);
    test_floor->fill_path(9, 22, 11, 28);
    test_floor->fill_path(7, 24, 13, 26);
    test_floor->set(5, 5, tile_type::mosaic_a);
    test_floor->set(5, 7, tile_type::mosaic_a);
    test_floor->set(7, 5, tile_type::mosaic_a);
    test_floor->set(7, 7, tile_type::mosaic_a);
    test_floor->set(13, 5, tile_type::mosaic_b);
    test_floor->set(13, 7, tile_type::mosaic_b);
    test_floor->set(15, 5, tile_type::mosaic_b);
    test_floor->set(15, 7, tile_type::mosaic_b);
    test_floor->set(11, 5, tile_type::block_a);
    test_floor->soften();
*/
    test_floor->load("data\\level.txt");

    portals.push_back(new portal(my_sdl, 10, 25, 0, 1));
    portals.push_back(new portal(my_sdl, 2, 2, 0, 2));

    player = new chara(my_sdl, "data\\chara.bmp", TILE_SIZE, 10, 6);
    // player->enqueue_anim(34);
    // objects.push_back(player);

    while(!stop) {
        // Draw
        test_floor->draw(my_sdl, tilesets.at(left_tileset));
        test_floor->draw(my_sdl, tilesets.at(right_tileset), SCREEN_W/2, 0);
        /*
        for(struct object * obj : objects) {
            obj->draw(my_sdl, test_tileset_01);
            obj->draw(my_sdl, test_tileset_02, SCREEN_W/2, 0);
        }
        */
        for(struct portal * portal : portals) {
            portal->draw(my_sdl, left_tileset);
            portal->draw(my_sdl, SCREEN_W/2, 0, right_tileset);
        }
        player->draw(my_sdl);
        player->draw(my_sdl, SCREEN_W/2, 0);

        // Update
        my_sdl->next_frame();

        // Process Input
        if(my_sdl->key(SDL_SCANCODE_ESCAPE)) {
            stop = true;
        } else {
            if(player->move(my_sdl, test_floor)) {
                for(struct portal * portal : portals) {
                    portal->step_on(player->x, player->y, &left_tileset);
                    portal->step_on(player->x, player->y, &right_tileset);
                }
            }
            if(my_sdl->keydown(SDL_SCANCODE_Q)) {
                left_tileset = (left_tileset+1)%tilesets.size();
            }
            if(my_sdl->keydown(SDL_SCANCODE_W)) {
                right_tileset = (right_tileset+1)%tilesets.size();
            }
        }
    }

    // Clean
    for(auto it : tilesets) {
        delete(it);
    }
    for(auto it : portals) {
        delete(it);
    }
    delete(test_floor);
    delete(player);
    /*
    for(auto it : objects) {
        delete(it);
    }
    */
    delete(my_sdl);
    return(EXIT_SUCCESS);
}
