#include "sdl.h"

#include "sprite.h"

#include <iostream>
#include <cstdlib>

sdl::sdl(int width, int height) :
    window(NULL),
    renderer(NULL),
    keyboard(NULL),
    max_key(0),
    width(width),
    height(height),
    scancodes(),
    sprites()
{
    // Initialize SDL video.
    if(SDL_Init(SDL_INIT_VIDEO /*|SDL_INIT_AUDIO*/) < 0) {
        this->error("Unable to init SDL");
    }

    // Create window.
    window = SDL_CreateWindow(
            NULL,
            SDL_WINDOWPOS_CENTERED,
            SDL_WINDOWPOS_CENTERED,
            0, 0, 0);
    if(!window)
        this->error("Unable to open new window");

    // Create renderer.
    renderer = SDL_CreateRenderer(window, -1, 0);
    if(!renderer)
        this->error("Unable to create new renderer");

    // Configure window.
    SDL_RenderSetLogicalSize(renderer, this->width, this->height);
    SDL_SetWindowSize(window, this->width, this->height);
    SDL_SetWindowPosition(window, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED);
    SDL_ShowCursor(SDL_DISABLE);
#ifndef DEBUG
    SDL_SetWindowFullscreen(this->window, SDL_WINDOW_FULLSCREEN);
#endif // DEBUG
    // SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "linear");
    // makes the scaled rendering look smoother. // DON'T DO THAT!!!

    // Store keyboard address.
    this->keyboard = SDL_GetKeyboardState(&(this->max_key));

    // First clear and present.
    SDL_SetRenderDrawColor(this->renderer, 0, 0, 0, 255);
    SDL_RenderClear(this->renderer);
    SDL_RenderPresent(this->renderer);
}

sdl::~sdl() {
    SDL_DestroyRenderer(this->renderer);
    SDL_DestroyWindow(this->window);
    SDL_Quit();
}

void sdl::error(std::string message) {
    std::cerr << message << " : " << SDL_GetError() << std::endl;
    exit(1);
}

void sdl::set_icon(std::string filename) {
    SDL_Surface* icon = SDL_LoadBMP(filename.c_str());
    if(!icon)
        this->error("Unable to load icon file " + filename);
    SDL_SetWindowIcon(this->window, icon);
    SDL_FreeSurface(icon);
}

void sdl::set_title(std::string title) {
    SDL_SetWindowTitle(this->window, title.c_str());
}

void sdl::next_frame() {
    SDL_Event event;

    SDL_RenderPresent(this->renderer);
    SDL_SetRenderDrawColor(this->renderer, 0, 0, 0, 255);
    SDL_RenderClear(this->renderer);
    SDL_Delay(10);

    this->scancodes.clear();
    while(SDL_PollEvent(&event)) {
        if(event.type == SDL_KEYDOWN) {
            this->scancodes.push_back(event.key.keysym.scancode);
        }
    }
}

bool sdl::key(SDL_Scancode key) {
    if(key < this->max_key) {
        return(this->keyboard[key] == 1);
    } else {
        return(false);
    }
}

bool sdl::keydown(SDL_Scancode key) {
    for(SDL_Scancode it : this->scancodes) {
        if(it == key) {
            return(true);
        }
    }
    return(false);
}

void sdl::load_sprite(std::string sprite_name, std::string file_name) {
    if(this->sprites[sprite_name] != NULL) {
        delete(this->sprites[sprite_name]);
    }
    this->sprites[sprite_name] = new sprite(this, file_name);
}

class sprite * sdl::get_sprite(std::string name) {
    class sprite * sprite;
    sprite = this->sprites[name];
    if(sprite == NULL) {
        this->load_sprite(name, "data\\"+name+".bmp");
        sprite = this->sprites[name];
        if(sprite == NULL) {
            std::cerr << "Sprite '" << name << "' doesn't exist." << std::endl;
            exit(1);
        }
    }
    return(sprite);
}
