#include "sprite.h"

#include "sdl.h"

extern void error(std::string msg);

sprite::sprite(class sdl * sdl, const std::string filename) {
    SDL_Surface * surface = SDL_LoadBMP(filename.c_str());
    if(surface == NULL)
        sdl->error("Unable to load bmp file");
    this->texture = SDL_CreateTextureFromSurface(sdl->renderer, surface);
    if(texture == NULL)
        sdl->error("Unable to create texture from surface");
    this->w = surface->w;
    this->h = surface->h;
    SDL_FreeSurface(surface);
}

sprite::sprite(class sdl * sdl, SDL_Surface * surface) {
    this->texture = SDL_CreateTextureFromSurface(sdl->renderer, surface);
    if(texture == NULL)
        sdl->error("Unable to create texture from surface");
    this->w = surface->w;
    this->h = surface->h;
}

sprite::~sprite() {
    SDL_DestroyTexture(this->texture);
}

void sprite::draw(class sdl * sdl, int x, int y) {
    SDL_Rect rect;
    rect.w = this->w;
    rect.h = this->h;
    rect.x = x;
    rect.y = y;
    SDL_RenderCopy(sdl->renderer, this->texture, NULL, &rect);
}

void sprite::draw_center(class sdl * sdl, float x, float y) {
    this->draw(sdl, x - this->w/2, y - this->h/2);
}
